(***********************************************************)
(*  This file is distributed under the terms of the        *)
(*    GNU Lesser General Public License version 3          *)
(*                                                         *)
(*          Copyright 2010: Damien Pous.                   *)
(*                                                         *)
(***********************************************************)


(* printing a list, with a given separator *)
let print_list sep print_elt = 
  let rec print f = function
    | [] -> ()
    | a::q -> Printf.fprintf f "%s%a%a" sep print_elt a print q
  in fun f -> function
    | [] -> ()
    | a::q -> Printf.fprintf f "%a%a" print_elt a print q


(* statistics: mean and standard deviation of an array *)
let sqr x = x*.x
let mean_stddev t =
  let n = Array.length t in
  let n' = float_of_int n in
  let rec sum acc n = if n<0 then acc else sum (acc +. t.(n)) (n-1) in
  let m = sum 0.0 (n-1) /. n' in
  let rec dev acc n = if n<0 then acc else sum (acc +. sqr (t.(n)-.m)) (n-1) in
  let e = sqrt (dev 0.0 (n-1) /. (n' -. 1.)) in (* n-1 for corrected empiric deviation *)
    m,e


(* timing: time required by a function *)
let time f x =
  let t0 = Sys.time() in
  let res = f x in
    Sys.time() -. t0, res


(* running a function for at most a given period of time *)
exception Timeout
let timeout tsecs f x =
  let handle_sigalrm signo = raise Timeout in
  let set_timer tsecs = 
    ignore (Unix.setitimer Unix.ITIMER_REAL { Unix.it_interval = 0.0; Unix.it_value = tsecs }) 
  in
  let oldsig = Sys.signal Sys.sigalrm (Sys.Signal_handle handle_sigalrm) in
    set_timer tsecs;
    try
      let res = f x in
	set_timer 0.0;
	Sys.set_signal Sys.sigalrm oldsig;
	Some res
    with Timeout ->
      Sys.set_signal Sys.sigalrm oldsig;
      None
