(***********************************************************)
(*  This file is distributed under the terms of the        *)
(*    GNU Lesser General Public License version 3          *)
(*                                                         *)
(*          Copyright 2010: Damien Pous.                   *)
(*                                                         *)
(***********************************************************)

let options = ref []
let toprint = ref []
let add_option ?(p=true) opt dsc knd fmt r =
  options := ("-"^opt, knd, dsc) :: !options;
  if p then toprint := (fun f -> Printf.fprintf f ("%s="^^fmt) opt !r) :: !toprint;
  r

let int_option ?p opt dsc default =
  let r = ref default in 
    add_option ?p opt dsc (Arg.Set_int r) "%i" r

let float_option ?p opt dsc default =
  let r = ref default in 
    add_option ?p opt dsc (Arg.Set_float r) "%.3f" r

let str_option ?p opt dsc default =
  let r = ref default in 
    add_option ?p opt dsc (Arg.Set_string r) "%s" r

let set_option ?p opt dsc =
  let r = ref false in 
    add_option ?p opt dsc (Arg.Set r) "%b" r

let clear_option ?p opt dsc =
  let r = ref true in
    add_option ?p opt dsc (Arg.Clear r) "%b" r

let print () =
  Printf.printf "# params: %a\n%!" (Common.print_list ", " (fun f p -> p f)) !toprint

let number     = int_option   "n" "nat number of generated expressions" 100
let p_constant = float_option "pcon" "float probability of constants w.r.t. variables" 0.2
let mll        = set_option   "mll"  " generate MLL expressions rather than MaLL expressions"

let () = 
  Random.init 0;
  let r = ref 0 in 
    options := ("-seed", Arg.Int (fun i -> r:= i; Random.init i), "int random seed") :: !options;
    toprint := (fun f -> Printf.fprintf f ("seed=%i") !r) :: !toprint



(* parsing simple integer list expressions *)

let rec seq s m n =
  if n=m then [s*n] 
  else if m<n then s*m :: seq s (m+1) n
  else s*m :: seq s (m-1) n

let rec queue f = 
  try Scanf.bscanf f ",%i%r" queue (fun x q -> x::q)
  with _ -> []

let intlist_of_string ~msg s =
  try Scanf.sscanf s "%i:%i..%i" seq with _ ->  
  try Scanf.sscanf s "%i..%i" (seq 1) with _ ->  
  try Scanf.sscanf s "%i%r" queue (fun x q -> x::q) with _ ->
  raise (Arg.Bad ("./utas: invalid interval for "^msg^"("^s^")"))

(*
let _ = set_of_string "3"
let _ = set_of_string "3..5"
let _ = set_of_string "5..3"
let _ = set_of_string "2:3..5"
let _ = set_of_string "2,3,5,7"
let _ = set_of_string "2,3,5,7 hop"
let _ = set_of_string ""
let _ = set_of_string " hip"
*)

let ilist_option opt dsc default =
  let r = ref default in 
    options := ("-"^opt, Arg.String (fun s -> r := intlist_of_string opt s), dsc) :: !options;
    r
