(***********************************************************)
(*  This file is distributed under the terms of the        *)
(*    GNU Lesser General Public License version 3          *)
(*                                                         *)
(*          Copyright 2010: Damien Pous.                   *)
(*                                                         *)
(***********************************************************)

(* current list of command line options *)
val options : (string * Arg.spec * string) list ref

(* adding options of the given type (the boolean p indicates whether
   we print the parameter in the function [print] below) *)
val int_option : ?p:bool -> string -> string -> int -> int ref
val float_option : ?p:bool -> string -> string -> float -> float ref
val str_option : ?p:bool -> string -> string -> string -> string ref
val set_option : ?p:bool -> string -> string -> bool ref
val clear_option : ?p:bool -> string -> string -> bool ref

(* printing the current set of parameters *)
val print : unit -> unit


(* parsing a list of integers from a string 
   valid integer sets are "7" "{2,3,5}" "3..5" "5..2" "3:1..4",
   the latter respectively denote the lists [3;4;5], [5;4;3], and [3;6;9;12]
   [msg] is used to display error messages
*)
val intlist_of_string: msg:string -> string -> int list

(* corresponding `integer list' option :  *)
val ilist_option : string -> string -> int list -> int list ref


(* base parameters *)
val p_constant : float ref
val mll : bool ref
val number : int ref
