#!/bin/sh

# default gnuplot terminal
TERM=""

ERR=""
CUM=""
WRT=""
WIDTH=1

if [ -z "$SIZE" ]; then 
    SIZE=7.5cm
fi

while true; do 
    case "$1" in
	-pdf) WIDTH=2; TERM="set terminal pdf dashed fname \"Times\" fsize 7 size $SIZE,6cm";; 
	-jpg) TERM="set terminal jpeg font \"/Library/Fonts/Arial.ttf\"";; 
	-term) shift; TERM="set terminal $1";; 
	-err) ERR=true;;
	-cum) CUM=true;;
	-size) WRT="-size";; 
	-vars) WRT="-vars";; 
	*) break;;
    esac
    shift
done

if ! [ -f "$1" -a -z "$3" ]; then 
    echo "usage: plot [-pdf|-jpg|-term term|-vars|-size|-cum|-err] <file1> [<file2>]" >&2
    exit 1
fi

export $(sed -n "s/, / /g;s/# params://p" $1)
CF=violet
CO=red
    
if [ -f "$2" ]; then
    hA=$(sed -n 's/opt=true/opt=false/g;s/#//p' $1)
    hB=$(sed -n 's/opt=true/opt=false/g;s/#//p' $2)
    if ! [ "$hA" = "$hB" ]; then 
	echo "file mismatch" >&2
	exit 1
    fi
    
    if [ "$opt" = true ]; then 
	FO=$1; FF=$2
    else
	FF=$1; FO=$2
    fi
elif [ -z "$2" ]; then
    FF=$1
else
    echo "usage: plot [-pdf|-jpg|-term term|-vars|-size|-cum|-err] <file1> [<file2>]" >&2
    exit 1
fi

V=$(tail -1 $FF | awk '{print $1}') 
S=$(tail -1 $FF | awk '{print $2}') 

MODE=$(head -2 $FF | tail -1 | sed 's/# //g')

case "$MODE$WRT" in
    timings-vars)
	cat <<EOF 
$TERM
set key left top
set logscale y
set title "Timings (mean over $n sequents with $S leaves)"
set xlabel "variables"
set ylabel "time (s)"
set grid
set style line 1 linetype 1 linewidth $WIDTH linecolor rgb "$CF"
set style line 2 linetype 2 linewidth $WIDTH linecolor rgb "$CO"
plot "<awk '{if (\$2==$S) print \$1,\$3}' $FF" with lines ls 1 title "without opt."\\
   , "<awk '{if (\$2==$S) print \$1,\$3}' $FO" with lines ls 2 title "with opt."\\
EOF
        [ -n "$ERR" ] && cat <<EOF
   , "<awk '{if (\$2==$S) print \$1,\$3,\$4}' $FF" with yerrorbars lc rgb "$CF" notitle\\
   , "<awk '{if (\$2==$S) print \$1,\$3,\$4}' $FO" with yerrorbars lc rgb "$CO" notitle
EOF
	;;

    timings-size)
	cat <<EOF 
$TERM
set key left top
set logscale y
set title "Timings (mean over $n sequents with $V variables)"
set xlabel "size (leaves)"
set ylabel "time (s)"
set grid
set style line 1 linetype 1 linewidth $WIDTH linecolor rgb "$CF"
set style line 2 linetype 2 linewidth $WIDTH linecolor rgb "$CO"
plot "<awk '{if (\$1==$V) print \$2,\$3}' $FF" with lines ls 1 title "without opt."\\
   , "<awk '{if (\$1==$V) print \$2,\$3}' $FO" with lines ls 2 title "with opt."\\
EOF
        [ -n "$ERR" ] && cat <<EOF
   , "<awk '{if (\$1==$V) print \$2,\$3,\$4}' $FF" with yerrorbars ls 1 notitle\\
   , "<awk '{if (\$1==$V) print \$2,\$3,\$4}' $FO" with yerrorbars ls 2 notitle
EOF
	;;

    proportions-vars)
	cat <<EOF 
$TERM
set title "Proportion of square sequents (over $n sequents with $S leaves)"
set xlabel "variables"
set ylabel "proportion (%)"
set yrange [0:100]
set grid
plot "$FF" using 1:3 with lines linecolor rgb "blue" notitle
EOF
	;;

    proportions-size)
	cat <<EOF 
$TERM
set title "Proportion of square sequents (over $n sequents with $V variables)"
set xlabel "size (leaves)"
set ylabel "proportion (%)"
set yrange [0:100]
set grid
plot "$FF" using 2:3 with lines linecolor rgb "blue" notitle
EOF
	;;

    correlation)
	cat <<EOF 
$TERM
set key left top
set logscale x
set logscale y
set grid
set title "Correlation between synchronous calls and elapsed time\n($n sequents with $S leaves and $V variables)"
set xlabel "recursive synchronous calls"
set ylabel "time (s)"
plot "$FF" using 3:4 lc rgb "$CF" title "without opt."\\
   , "$FO" using 3:4 lc rgb "$CO" title "with opt."
EOF
	;;

    "time distribution")
	if [ "$CUM" = true ]; then 
	    T="requiring less than"
	    C=5
	    K="set key right bottom"
	    M=100
	else
	    T="yielding"
	    C=4
	    K=""
	    M=30
	fi
	cat <<EOF 
$TERM
$K
set logscale x
# set logscale y
set grid
set title "Proportion of sequents $T a given amount of time\n(over $n sequents with $S leaves and $V variables)"
set ylabel "proportion (%)"
set xlabel "time (s)"
set style line 1 linetype 1 linewidth $WIDTH linecolor rgb "$CF"
set style line 2 linetype 2 linewidth $WIDTH linecolor rgb "$CO"
plot "$FF" using 3:$C with lines ls 1 title "without opt."\\
   , "$FO" using 3:$C with lines ls 2 title "with opt."\\
   , "<awk '{if (int (\$5 * 10000) >=1000000) { print \$3,0; print \$3,$M } }' $FF" with lines ls 1 notitle \\
   , "<awk '{if (int (\$5 * 10000) >=1000000) { print \$3,0; print \$3,$M } }' $FO" with lines ls 2 notitle
EOF
	;;

    "calls distribution")
	if [ "$CUM" = true ]; then 
	    T="requiring less than"
	    C=5
	    K="set key right bottom"
	    M=100
	else
	    T="yielding"
	    C=4
	    K=""
	    M=30
	fi
	cat <<EOF 
$TERM
$K
set logscale x
set grid
set title "Proportion of sequents $T a given number of calls\n(over $n sequents with $S leaves and $V variables)"
set ylabel "proportion (%)"
set xlabel "calls"
set style line 1 linetype 1 linewidth $WIDTH linecolor rgb "$CF"
set style line 2 linetype 2 linewidth $WIDTH linecolor rgb "$CO"
plot "$FF" using 3:$C with lines ls 1 title "without opt."\\
   , "$FO" using 3:$C with lines ls 2 title "with opt."\\
   , "<awk '{if (int (\$5 * 10000) >=1000000) { print \$3,0; print \$3,$M } }' $FF" with lines ls 1 notitle \\
   , "<awk '{if (int (\$5 * 10000) >=1000000) { print \$3,0; print \$3,$M } }' $FO" with lines ls 2 notitle
EOF
	;;

    sized-calls)
	cat <<EOF 
$TERM
set grid
# set logscale y
set title "Recursive synchronous calls, indexed by sequents size\n(over $n sequents with $S leaves and $V variables)"
set ylabel "recursive synchronous calls"
set xlabel "size (leaves)"
set style line 1 linetype 1 linewidth $WIDTH linecolor rgb "$CO"
set style line 2 linetype 2 linewidth $WIDTH linecolor rgb "green"
plot \\
     "$FF" using 3:4 with lines ls 1 title "calls", \\
     "$FF" using 3:5 with lines ls 2 title "skipped calls"
EOF
	;;

    timings) echo "please use [-vars v] or [-size s] to plot timings" >&2; exit 1;;

    *) echo "unrecognized file format ($MODE) or invalid use of [-vars|-size]" >&2; exit 1;;
esac | gnuplot

