(***********************************************************)
(*  This file is distributed under the terms of the        *)
(*    GNU Lesser General Public License version 3          *)
(*                                                         *)
(*          Copyright 2010: Damien Pous.                   *)
(*                                                         *)
(***********************************************************)

open Formula

let choice2 a b = if Random.bool () then a else b
let choice3 a b c = match Random.int 3 with 0 -> a | 1 -> b | _ -> c
let choice4 a b c d = match Random.int 4 with 0 -> a | 1 -> b | 2 -> c | _ -> d

let random_const () = choice2 bot one

let random_var v =
  if v = 0 then random_const() 
  else let i = Random.int v in choice2 (var i) (rav i)

let random_leaf v =
  if Random.float 1. < !Param.p_constant then random_const () else random_var v

let discr a = match a with 
  | `One -> `One
  | `Seq l when Seq.is_empty l -> `Bot
  | _ -> `Som a

(* we generate expressions in normal form (i.e., such that a\otime 1
   and a\parr\bot are always reduced), with the exact specified number
   of leaves *)

let rec mll_formula v = function
  | 1 -> random_leaf v
  | n -> let l = 1+Random.int (n-1) in
      match discr (mll_formula v l), discr (mll_formula v (n-l)) with
	| `One  ,`Bot   -> if v=0 then par one one else par (random_var v) (random_var v)
	| `Bot  ,`One   -> if v=0 then ten bot bot else ten (random_var v) (random_var v)
	| `One  ,`One   -> par one one
	| `Bot  ,`Bot   -> ten bot bot
	| `One  ,`Som b -> par one b
	| `Som a,`One   -> par a one
	| `Bot  ,`Som b -> ten bot b
	| `Som a,`Bot   -> ten a bot
	| `Som a,`Som b -> choice2 ten par a b

let rec mall_formula v = function
  | 1 -> random_leaf v
  | n -> let l = 1+Random.int (n-1) in
      match discr (mall_formula v l), discr (mall_formula v (n-l)) with
	| `One  ,`Bot   -> choice2 wit opl         one bot
	| `Bot  ,`One   -> choice2 wit opl         bot one
	| `One  ,`One   -> choice3 wit opl par     one one
	| `Bot  ,`Bot   -> choice3 wit opl ten     bot bot
	| `One  ,`Som b -> choice3 wit opl par     one b
	| `Som a,`One   -> choice3 wit opl par     a one
	| `Bot  ,`Som b -> choice3 wit opl ten     bot b
	| `Som a,`Bot   -> choice3 wit opl ten     a bot
	| `Som a,`Som b -> choice4 wit opl ten par a b

let formula v n = if !Param.mll then mll_formula v n else mall_formula v n
let sequent v n = singleton (formula v n)
