(***********************************************************)
(*  This file is distributed under the terms of the        *)
(*    GNU Lesser General Public License version 3          *)
(*                                                         *)
(*          Copyright 2010: Damien Pous.                   *)
(*                                                         *)
(***********************************************************)

(* various hand-written tests *)

open Formula

let a = var 0
let b = var 1
let c = var 2
let a' = rav 0
let b' = rav 1
let c' = rav 2
let variables = 3
let ( $ ) = par
let ( + ) = opl
let ( & ) = wit
let ( * ) = ten
let ld a b = dual a $ b
let rd a b = a $ dual b
let ( / ) = rd
let leq a b = [dual a;b]

let map = List.map (fun (b,l) -> b,seq_of_list l)

let mll_provables = map [
  true,  [a';a];
  true,  [c;c'];
  true,  [bot;a;a'];
  true,  [b';b*bot;a;a'];
  true,  [a';b';b*bot;a];
  true,  leq a a;
  false, leq a b;
  true,  leq (a*b) (a*b);
  true,  leq (a*b*c) (a*b*c);
  true,  leq (a*(b*c)) ((a*b)*c);
  true,  leq (a*(b*one*c)) ((a*one*one*b)*c);
  true,  leq (a/a) (a/a);
  false, leq (a/a) (a/b);
  true,  leq (a/a*b) (a/a*b);
  true,  leq ((a/(b*c))*(b*c)) (a);
  false, leq ((a/(b*c))*(c*b)) (a);
  true,  leq (ld a (b/c)) ((ld a b)/c);
  true,  leq (a/ld (a/b) a) (a/b);
  true,  leq (a/b) (a/ld (a/b) a) ;
  false, leq (a/ld c a) c;
  true,  leq ((one/a)*a) ((one*a)/a);
]

let provables = mll_provables @ map [
  true,  [(one+a)*a;a';bot&a'];
  true,  leq a (a+b);
  false, leq (a+b) a;
  true,  leq b (a+b+c);
  true,  leq (a+b) (b+a+c);
  true,  leq ((a+b)*c) (a*c+b*c);
  true,  leq (a*c+b*c) ((a+b)*c);
  true,  leq (a&b) b;
  false, leq (a&b) (b&c);
  true,  leq (a&b) (b+a);
  false, leq (a+b) (b&a);
  false, leq (a+b) a;
  false, leq (a+b) (a+c);
  false, leq (a*c&b*c) ((a&b)*c);
  true,  leq ((one&b)*c) c;
  true,  leq ((a'&b')*c') (a'*c');
  true,  leq ((a&b)*c) (a*c);
  true,  leq ((a&b)*c) (a*c&b*c);
]

let squares = List.filter fst provables @ map [
  true,  [a*a];
  true,  [a;a];
  true,  [a'*b*a];
  true,  [(a'$b')*a];
  true,  [a;a';b;a;a';b'];
  true,  [a;a';b*a*a';b'];
  false, [a];
  true,  [a;b;c;a'];
  false, [b;c;a';a];
  true,  [a;a';a'];
  false, [a';a;a'];
  true,  [a;(b*b');a';a'];
  true,  [a;one;a';a'];
]

let check msg f k x =
  List.iter 
    (fun (v,s) -> 
       let w = f s in
	 if v <> w then
	   (Printf.fprintf stderr "sanity check failed: %b <> %s %a\n%!" v msg print_seq s; k x)
    )
